@@ -21,11 +21,11 @@ import sys
 import zlib
 
 from mutagen import FileType
-from mutagen._util import cdata, resize_bytes, MutagenError
+from mutagen._util import cdata, resize_bytes, MutagenError, loadfile, seek_end
 from ._compat import cBytesIO, reraise, chr_, izip, xrange
 
 
-class error(IOError, MutagenError):
+class error(MutagenError):
     """Ogg stream parsing errors."""
 
     pass
@@ -42,14 +42,15 @@ class OggPage(object):
     to the start of the next page.
 
     Attributes:
-
-    * version -- stream structure version (currently always 0)
-    * position -- absolute stream position (default -1)
-    * serial -- logical stream serial number (default 0)
-    * sequence -- page sequence number within logical stream (default 0)
-    * offset -- offset this page was read from (default None)
-    * complete -- if the last packet on this page is complete (default True)
-    * packets -- list of raw packet data (default [])
+        version (`int`): stream structure version (currently always 0)
+        position (`int`): absolute stream position (default -1)
+        serial (`int`): logical stream serial number (default 0)
+        sequence (`int`): page sequence number within logical stream
+            (default 0)
+        offset (`int` or `None`): offset this page was read from (default None)
+        complete (`bool`): if the last packet on this page is complete
+            (default True)
+        packets (List[`bytes`]): list of raw packet data (default [])
 
     Note that if 'complete' is false, the next page's 'continued'
     property must be true (so set both when constructing pages).
@@ -67,6 +68,8 @@ class OggPage(object):
     complete = True
 
     def __init__(self, fileobj=None):
+        """Raises error, IOError, EOFError"""
+
         self.packets = []
 
         if fileobj is None:
@@ -439,14 +442,15 @@ class OggPage(object):
 
         This finds the last page in the actual file object, or the last
         page in the stream (with eos set), whichever comes first.
+
+        Returns None in case no page with the serial exists.
+        Raises error in case this isn't a valid ogg stream.
+        Raises IOError.
         """
 
         # For non-muxed streams, look at the last page.
-        try:
-            fileobj.seek(-256 * 256, 2)
-        except IOError:
-            # The file is less than 64k in length.
-            fileobj.seek(0)
+        seek_end(fileobj, 256 * 256)
+
         data = fileobj.read()
         try:
             index = data.rindex(b"OggS")
@@ -484,65 +488,92 @@ class OggPage(object):
 
 
 class OggFileType(FileType):
-    """An generic Ogg file."""
+    """OggFileType(filething)
+
+    An generic Ogg file.
+
+    Arguments:
+        filething (filething)
+    """
 
     _Info = None
     _Tags = None
     _Error = None
     _mimes = ["application/ogg", "application/x-ogg"]
 
-    def load(self, filename):
-        """Load file information from a filename."""
+    @loadfile()
+    def load(self, filething):
+        """load(filething)
 
-        self.filename = filename
-        with open(filename, "rb") as fileobj:
-            try:
-                self.info = self._Info(fileobj)
-                self.tags = self._Tags(fileobj, self.info)
-                self.info._post_tags(fileobj)
-            except error as e:
-                reraise(self._Error, e, sys.exc_info()[2])
-            except EOFError:
-                raise self._Error("no appropriate stream found")
+        Load file information from a filename.
 
-    def delete(self, filename=None):
-        """Remove tags from a file.
-
-        If no filename is given, the one most recently loaded is used.
+        Args:
+            filething (filething)
+        Raises:
+            mutagen.MutagenError
         """
 
-        if filename is None:
-            filename = self.filename
+        fileobj = filething.fileobj
+
+        try:
+            self.info = self._Info(fileobj)
+            self.tags = self._Tags(fileobj, self.info)
+            self.info._post_tags(fileobj)
+        except (error, IOError) as e:
+            reraise(self._Error, e, sys.exc_info()[2])
+        except EOFError:
+            raise self._Error("no appropriate stream found")
+
+    @loadfile(writable=True)
+    def delete(self, filething):
+        """delete(filething=None)
+
+        Remove tags from a file.
+
+        If no filename is given, the one most recently loaded is used.
+
+        Args:
+            filething (filething)
+        Raises:
+            mutagen.MutagenError
+        """
+
+        fileobj = filething.fileobj
 
         self.tags.clear()
         # TODO: we should delegate the deletion to the subclass and not through
         # _inject.
-        with open(filename, "rb+") as fileobj:
+        try:
             try:
                 self.tags._inject(fileobj, lambda x: 0)
             except error as e:
                 reraise(self._Error, e, sys.exc_info()[2])
             except EOFError:
                 raise self._Error("no appropriate stream found")
+        except IOError as e:
+            reraise(self._Error, e, sys.exc_info()[2])
 
     def add_tags(self):
         raise self._Error
 
-    def save(self, filename=None, padding=None):
-        """Save a tag to a file.
+    @loadfile(writable=True)
+    def save(self, filething, padding=None):
+        """save(filething=None, padding=None)
+
+        Save a tag to a file.
 
         If no filename is given, the one most recently loaded is used.
+
+        Args:
+            filething (filething)
+            padding (PaddingFunction)
+        Raises:
+            mutagen.MutagenError
         """
 
-        if filename is None:
-            filename = self.filename
-        fileobj = open(filename, "rb+")
         try:
-            try:
-                self.tags._inject(fileobj, padding)
-            except error as e:
-                reraise(self._Error, e, sys.exc_info()[2])
-            except EOFError:
-                raise self._Error("no appropriate stream found")
-        finally:
-            fileobj.close()
+            self.tags._inject(filething.fileobj, padding)
+        except (IOError, error) as e:
+            reraise(self._Error, e, sys.exc_info()[2])
+        except EOFError:
+            raise self._Error("no appropriate stream found")
